#if !defined(XmlRpcConnection_hxx)
#define XmlRpcConnection_hxx 

#include <atomic>
#include <map>
#include <rutil/Data.hxx>
#include <rutil/Socket.hxx>
#include <rutil/TransportType.hxx>
#include <resip/stack/Tuple.hxx>

#include "repro/XmlRpcServerBase.hxx"

namespace repro
{

class XmlRpcConnectionBase
{
public:
   XmlRpcConnectionBase(XmlRpcServerBase& server);
   virtual ~XmlRpcConnectionBase();

   unsigned int getConnectionId() const { return mConnectionId; }
   unsigned int getNextRequestId() { return mNextRequestId++; }

   virtual void buildFdSet(resip::FdSet& fdset) = 0;
   virtual bool process(resip::FdSet& fdset) = 0;

   virtual bool sendResponse(unsigned int requestId, const resip::Data& responseData, bool isFinal);
   virtual void sendEvent(const resip::Data& eventData) = 0;

protected:
   virtual void sendResponse(const resip::Data& response) = 0;

   XmlRpcServerBase& mXmlRcpServer;

   typedef std::map<unsigned int, resip::Data> RequestMap;
   RequestMap mRequests;

private:
   const unsigned int mConnectionId;
   static std::atomic<unsigned int> NextConnectionId;
   unsigned int mNextRequestId;
};

class XmlRpcSocketConnection : public XmlRpcConnectionBase
{
   friend class XmlRpcServerBase;

public:
   XmlRpcSocketConnection(XmlRpcSocketServer& server, resip::Socket sock);
   virtual ~XmlRpcSocketConnection();

   virtual void buildFdSet(resip::FdSet& fdset);
   virtual bool process(resip::FdSet& fdset);

   virtual void sendEvent(const resip::Data& eventData);

protected:
   virtual void sendResponse(const resip::Data& response);

private:
   bool processSomeReads();
   bool processSomeWrites();
   bool tryParse(); // returns true if we processed something and there is more data in the buffer

   resip::Socket mSock;
   resip::Data mRxBuffer;
   resip::Data mTxBuffer;
};

#ifdef BUILD_QPID_PROTON

class XmlRpcProtonConnection : public XmlRpcConnectionBase
{
   friend class XmlRpcServerBase;

public:
   XmlRpcProtonConnection(XmlRpcProtonServer& server, const resip::Data& replyTo,
                          std::shared_ptr<resip::ProtonThreadBase::ProtonSenderBase> sender);
   virtual ~XmlRpcProtonConnection();

   void onRequest(unsigned int requestId, const resip::Data& request);

   virtual void buildFdSet(resip::FdSet& fdset);
   virtual bool process(resip::FdSet& fdset);

   virtual void sendEvent(const resip::Data& eventData);

protected:
   virtual void sendResponse(const resip::Data& response);

private:
   resip::Data mReplyTo;
   std::shared_ptr<resip::ProtonThreadBase::ProtonSenderBase> mSender;
};

#endif

}

#endif  

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * Copyright (c) 2010 SIP Spectrum, Inc.  All rights reserved.
 * Copyright (c) 2022 Daniel Pocock https://danielpocock.com
 * Copyright (c) 2022 Software Freedom Institute LLC https://softwarefreedom.institute
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
